==> mysql-workbench-gpl-5.2.29-src.tar.gz: mysql-workbench-gpl-5.2.29-src/backend/wbpublic/grtdb/diff_dbobjectmatch.h <==
/* 
 * © 2007-2008 MySQL AB, 2008-2009 Sun Microsystems, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef _DB_OBJECT_MATCH_H_
#define _DB_OBJECT_MATCH_H_


#include "grtpp_util.h"
#include "wbpublic_public_interface.h"

#include "string_utilities.h"
#include "grt/common.h"
#include "grtdb/charset_utils.h"
#include "grtsqlparser/sql_facade.h"
#include "db_object_helpers.h"

namespace grt
{

struct WBPUBLICBACKEND_PUBLIC_FUNC DbObjectMatchAlterOmf : public Omf
{
  virtual bool less(const ValueRef& , const ValueRef&) const;
  virtual bool equal(const ValueRef& , const ValueRef&) const;
};

struct WBPUBLICBACKEND_PUBLIC_FUNC DbObjectMatchRecreateOmf : public Omf
{
  virtual bool less(const ValueRef& , const ValueRef&) const;
  virtual bool equal(const ValueRef& , const ValueRef&) const;
};

  template <typename Tgrt>
  class NormalizedComparer
    {
    protected:
      Tgrt _grt;
    public:
      NormalizedComparer(Tgrt grt):_grt(grt){};
      bool normalizedComparison(const ValueRef obj1, const ValueRef obj2, const std::string name)const
      {
        if(name == "tableEngine")
        {
          std::string str1= ObjectRef::cast_from(obj1).get_string_member(name);
          std::string str2= ObjectRef::cast_from(obj2).get_string_member(name);
          base::toupper(str1);
          base::toupper(str2);
          return str1 == str2;       
        }

        if(((name == "name") || (name == "oldName")) && !(db_ColumnRef::can_wrap(obj1)))
        {
          std::string str1= ObjectRef::cast_from(obj1).get_string_member(name);
          std::string str2= ObjectRef::cast_from(obj2).get_string_member(name);
          base::toupper(str1);
          base::toupper(str2);
          return str1 == str2;       
        }

        if (name == "referencedTable")
        {
          std::string str1 = db_mysql_ForeignKeyRef::cast_from(obj1)->referencedTable()->name();
          std::string str2 = db_mysql_ForeignKeyRef::cast_from(obj2)->referencedTable()->name();
          base::toupper(str1);
          base::toupper(str2);
          return str1 == str2;              
        }
        if ((name == "rowFormat") || (name == "packKeys"))
        {
          std::string str1= ObjectRef::cast_from(obj1).get_string_member(name);
          std::string str2= ObjectRef::cast_from(obj2).get_string_member(name);
          base::toupper(str1);
          base::toupper(str2);
          if (str1 == "DEFAULT")
            str1 = "";
          if (str2 == "DEFAULT")
            str2 = "";
          return str1 == str2;       
        }
        if ((name == "characterSetName") || (name == "collationName"))
        {
          if (db_ColumnRef::can_wrap(obj1))
          {
            std::string fieldname(name == "characterSetName"?"defaultCharacterSetName":"defaultCollationName");
            std::string sql1= ObjectRef::cast_from(obj1).get_string_member(name);
            std::string sql2= ObjectRef::cast_from(obj2).get_string_member(name);
            if (sql1.empty())
              sql1 = GrtNamedObjectRef::cast_from(obj2)->owner().get_string_member(fieldname);
            if (sql2.empty())
              sql2 = GrtNamedObjectRef::cast_from(obj2)->owner().get_string_member(fieldname);
            return sql1 == sql2;
          }
          std::string sql1= ObjectRef::cast_from(obj1).get_string_member(name);
          std::string sql2= ObjectRef::cast_from(obj2).get_string_member(name);
          return sql1.empty() || sql2.empty();
        }
        if (name == "foreignKeys")
        {
          // Here we do not compare the ability for engines to support foreign keys but
          // check if at least one engine does not. This can be used to optimize further FK handling
          // (no need to check all the FKs then).
          grt::StringRef ename = db_mysql_TableRef::cast_from(obj1)->tableEngine();
          db_mysql_StorageEngineRef engine = bec::TableHelper::get_engine_by_name(_grt, ename);

          if (!engine.is_valid() || !engine->supportsForeignKeys())
            return true;

          ename = db_mysql_TableRef::cast_from(obj2)->tableEngine();
          engine = bec::TableHelper::get_engine_by_name(_grt, ename);

          if (!engine.is_valid() || !engine->supportsForeignKeys())
            return true;

          return false;
        }

        if ((name == "deleteRule") || (name == "updateRule"))
        {
          std::string sql1= ObjectRef::cast_from(obj1).get_string_member(name);
          std::string sql2= ObjectRef::cast_from(obj2).get_string_member(name);
          base::toupper(sql1);
          base::toupper(sql2);
          if (sql1 == "RESTRICT")
            sql1 = "";
          if (sql2 == "RESTRICT")
            sql2 = "";
          return sql1 == sql2;
        }
        
        if((name == "comment"))
        {
          std::string sql1= ObjectRef::cast_from(obj1).get_string_member(name);
          std::string sql2= ObjectRef::cast_from(obj2).get_string_member(name);
          return base::escape_sql_string(sql1) == base::escape_sql_string(sql2);
        }

        if((name == "formattedRawType")||(name == "formattedType"))
        {
        std::string sql1= ObjectRef::cast_from(obj1).get_string_member(name);
        std::string sql2= ObjectRef::cast_from(obj2).get_string_member(name);
        SqlFacade* parser = SqlFacade::instance_for_rdbms_name(_grt, "Mysql");
        
        if (!parser)
          return false;
        
        sql1= parser->remove_inter_token_spaces(sql1);
        sql2= parser->remove_inter_token_spaces(sql2);
        base::toupper(sql1);
        base::toupper(sql2);
//        if (sql1 != sql2)
//        std::cout<<"============"<<sql1<<std::endl<<std::endl<<sql2<<"==========================="<<std::endl;
        return sql1 == sql2;

        }
        if(name == "sqlDefinition")
        {
        std::string sql1= ObjectRef::cast_from(obj1).get_string_member(name);
        std::string sql2= ObjectRef::cast_from(obj2).get_string_member(name);
        SqlFacade* parser = SqlFacade::instance_for_rdbms_name(_grt, "Mysql");
        
        if (!parser)
          return false;
        std::string schema1 = db_TriggerRef::can_wrap(obj1)?
          GrtNamedObjectRef::cast_from(obj1)->owner()->owner()->name():
          GrtNamedObjectRef::cast_from(obj1)->owner()->name();
        std::string schema2 = db_TriggerRef::can_wrap(obj2)?
          GrtNamedObjectRef::cast_from(obj2)->owner()->owner()->name():
          GrtNamedObjectRef::cast_from(obj2)->owner()->name();
        sql1= parser->normalizeSqlStatement(sql1, schema1);
        sql2= parser->normalizeSqlStatement(sql2, schema2);
        base::toupper(sql1);
        base::toupper(sql2);
//        sql1.erase(std::remove(sql1.begin(), sql1.end(), '\n'), sql1.end());
//        sql2.erase(std::remove(sql2.begin(), sql2.end(), '\n'), sql2.end());
//        if (sql1 != sql2)
//        std::cout<<"\n=======================Def1=======================\n"<<sql1<<std::endl<<"\n=======================Def2=======================\n"<<std::endl<<sql2<<"!!!"<<(sql1 == sql2)<<std::endl;
        return sql1 == sql2;
        }
        return false; 
      };
      operator TSlotNormalizerSlot(){return sigc::mem_fun(*this, &NormalizedComparer<Tgrt>::normalizedComparison);};
    };
  
  
} // namespace grt

#endif // _DB_OBJECT_MATCH_H_

==> mysql-workbench-gpl-5.2.33-src.tar.gz: mysql-workbench-gpl-5.2.33-src/backend/wbpublic/grtdb/diff_dbobjectmatch.h <==
/* 
 * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of the
 * License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
#ifndef _DB_OBJECT_MATCH_H_
#define _DB_OBJECT_MATCH_H_

#include <boost/bind.hpp>

#include "grtpp_util.h"
#include "wbpublic_public_interface.h"

#include "base/string_utilities.h"
#include "grt/common.h"
#include "grtdb/charset_utils.h"
#include "grtsqlparser/sql_facade.h"
#include "db_object_helpers.h"

namespace grt
{

struct WBPUBLICBACKEND_PUBLIC_FUNC DbObjectMatchAlterOmf : public Omf
{
  virtual bool less(const ValueRef& , const ValueRef&) const;
  virtual bool equal(const ValueRef& , const ValueRef&) const;
};

struct WBPUBLICBACKEND_PUBLIC_FUNC DbObjectMatchRecreateOmf : public Omf
{
  virtual bool less(const ValueRef& , const ValueRef&) const;
  virtual bool equal(const ValueRef& , const ValueRef&) const;
};

  template <typename Tgrt>
  class NormalizedComparer
    {
    protected:
      Tgrt _grt;
    public:
      NormalizedComparer(Tgrt grt):_grt(grt){};
      bool normalizedComparison(const ValueRef obj1, const ValueRef obj2, const std::string name)const
      {

        if(name == "comment")
        {
          std::string str1= ObjectRef::cast_from(obj1).get_string_member(name);
          std::string str2= ObjectRef::cast_from(obj2).get_string_member(name);
          str1 = bec::TableHelper::get_sync_comment(str1);
          str2 = bec::TableHelper::get_sync_comment(str2);
          if (db_mysql_SchemaRef::can_wrap(obj1))
            return true;
          return str1 == str2;       
        }
        if(name == "tableEngine")
        {
          std::string str1= ObjectRef::cast_from(obj1).get_string_member(name);
          std::string str2= ObjectRef::cast_from(obj2).get_string_member(name);
          base::toupper(str1);
          base::toupper(str2);
          return str1 == str2;       
        }

        if(((name == "name") || (name == "oldName")) && !(db_ColumnRef::can_wrap(obj1)))
        {
          std::string str1= ObjectRef::cast_from(obj1).get_string_member(name);
          std::string str2= ObjectRef::cast_from(obj2).get_string_member(name);
          base::toupper(str1);
          base::toupper(str2);
          return str1 == str2;       
        }

        if (name == "referencedTable")
        {
          std::string str1 = db_mysql_ForeignKeyRef::cast_from(obj1)->referencedTable()->name();
          std::string str2 = db_mysql_ForeignKeyRef::cast_from(obj2)->referencedTable()->name();
          base::toupper(str1);
          base::toupper(str2);
          return str1 == str2;              
        }
        if ((name == "rowFormat") || (name == "packKeys"))
        {
          std::string str1= ObjectRef::cast_from(obj1).get_string_member(name);
          std::string str2= ObjectRef::cast_from(obj2).get_string_member(name);
          base::toupper(str1);
          base::toupper(str2);
          if (str1 == "DEFAULT")
            str1 = "";
          if (str2 == "DEFAULT")
            str2 = "";
          return str1 == str2;       
        }
        if ((name == "characterSetName") || (name == "collationName"))
        {
          if (db_ColumnRef::can_wrap(obj1))
          {
            std::string fieldname(name == "characterSetName"?"defaultCharacterSetName":"defaultCollationName");
            std::string sql1= ObjectRef::cast_from(obj1).get_string_member(name);
            std::string sql2= ObjectRef::cast_from(obj2).get_string_member(name);
            if (sql1.empty())
              sql1 = GrtNamedObjectRef::cast_from(obj2)->owner().get_string_member(fieldname);
            if (sql2.empty())
              sql2 = GrtNamedObjectRef::cast_from(obj2)->owner().get_string_member(fieldname);
            return sql1 == sql2;
          }
          std::string sql1= ObjectRef::cast_from(obj1).get_string_member(name);
          std::string sql2= ObjectRef::cast_from(obj2).get_string_member(name);
          return sql1.empty() || sql2.empty();
        }
        if (name == "foreignKeys")
        {
          // Here we do not compare the ability for engines to support foreign keys but
          // check if at least one engine does not. This can be used to optimize further FK handling
          // (no need to check all the FKs then).
          grt::StringRef ename = db_mysql_TableRef::cast_from(obj1)->tableEngine();
          db_mysql_StorageEngineRef engine = bec::TableHelper::get_engine_by_name(_grt, ename);

          if (!engine.is_valid() || !engine->supportsForeignKeys())
            return true;

          ename = db_mysql_TableRef::cast_from(obj2)->tableEngine();
          engine = bec::TableHelper::get_engine_by_name(_grt, ename);

          if (!engine.is_valid() || !engine->supportsForeignKeys())
            return true;

          return false;
        }

        if ((name == "deleteRule") || (name == "updateRule"))
        {
          std::string sql1= ObjectRef::cast_from(obj1).get_string_member(name);
          std::string sql2= ObjectRef::cast_from(obj2).get_string_member(name);
          base::toupper(sql1);
          base::toupper(sql2);
          if (sql1 == "RESTRICT")
            sql1 = "";
          if (sql2 == "RESTRICT")
            sql2 = "";
          return sql1 == sql2;
        }
        
        if((name == "comment"))
        {
          std::string sql1= ObjectRef::cast_from(obj1).get_string_member(name);
          std::string sql2= ObjectRef::cast_from(obj2).get_string_member(name);
          return base::escape_sql_string(sql1) == base::escape_sql_string(sql2);
        }

        if((name == "formattedRawType")||(name == "formattedType"))
        {
        std::string sql1= ObjectRef::cast_from(obj1).get_string_member(name);
        std::string sql2= ObjectRef::cast_from(obj2).get_string_member(name);
        SqlFacade* parser = SqlFacade::instance_for_rdbms_name(_grt, "Mysql");
        
        if (!parser)
          return false;
        
        sql1= parser->remove_inter_token_spaces(sql1);
        sql2= parser->remove_inter_token_spaces(sql2);
        base::toupper(sql1);
        base::toupper(sql2);
//        if (sql1 != sql2)
//        std::cout<<"============"<<sql1<<std::endl<<std::endl<<sql2<<"==========================="<<std::endl;
        return sql1 == sql2;

        }
        if(name == "sqlDefinition")
        {
        std::string sql1= ObjectRef::cast_from(obj1).get_string_member(name);
        std::string sql2= ObjectRef::cast_from(obj2).get_string_member(name);
        SqlFacade* parser = SqlFacade::instance_for_rdbms_name(_grt, "Mysql");
        
        if (!parser)
          return false;
        std::string schema1 = db_TriggerRef::can_wrap(obj1)?
          GrtObjectRef::cast_from(obj1)->owner()->owner()->name():
          GrtObjectRef::cast_from(obj1)->owner()->name();
        std::string schema2 = db_TriggerRef::can_wrap(obj2)?
          GrtObjectRef::cast_from(obj2)->owner()->owner()->name():
          GrtObjectRef::cast_from(obj2)->owner()->name();
        sql1= parser->normalizeSqlStatement(sql1, schema1);
        sql2= parser->normalizeSqlStatement(sql2, schema2);
        sql1= parser->remove_inter_token_spaces(sql1);
        sql2= parser->remove_inter_token_spaces(sql2);
        base::toupper(sql1);
        base::toupper(sql2);
//        sql1.erase(std::remove(sql1.begin(), sql1.end(), '\n'), sql1.end());
//        sql2.erase(std::remove(sql2.begin(), sql2.end(), '\n'), sql2.end());
//        if (sql1 != sql2)
//        std::cout<<"\n=======================Def1=======================\n"<<sql1<<std::endl<<"\n=======================Def2=======================\n"<<std::endl<<sql2<<"!!!"<<(sql1 == sql2)<<std::endl;
        return sql1 == sql2;
        }
        return false; 
      };
      operator TSlotNormalizerSlot(){return boost::bind(&NormalizedComparer<Tgrt>::normalizedComparison,*this, _1, _2, _3);};
        //sigc::mem_fun(*this, &NormalizedComparer<Tgrt>::normalizedComparison);};
    };
  
  
} // namespace grt

#endif // _DB_OBJECT_MATCH_H_
